//
//  ViewController.swift
//  ArduinoTemperatureSwift
//
//  Created by Tony Dahbura on 2/21/15.
//  Copyright (c) 2015 Tony Dahbura. All rights reserved.
//

import UIKit
import Foundation


let INVALID_TEMPERATURE : Float = -1000.0
let SETTINGS_FILENAME = "userSettings.plist"

class ViewController: UIViewController, SettingsViewControllerDelegate {
    
    
    @IBOutlet weak var interiorTemperature : UILabel!
    @IBOutlet weak var exteriorTemperature : UILabel!
    @IBOutlet weak var updateButton: UIButton!
    @IBOutlet weak var lastUpdatedLabel : UILabel!
    
    var indoorTemperatureCelsius = INVALID_TEMPERATURE
    var outdoorTemperatureCelsius = INVALID_TEMPERATURE
    var lastUpdate = NSDate()
    var theURL = ""
    var displayAsCelsius = true
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        loadSettings()
        updateDisplayWithCurrentReadings()
    }
    
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if (segue.identifier == "showAlternate") {
            let vc = segue.destinationViewController as! SettingsViewController
            vc.delegate = self
        }
    }
    
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    
    
    func settingsViewControllerDidFinish(controller: SettingsViewController) {
        
        self.dismissViewControllerAnimated(true, completion: nil)
        
        //save our settings - the flipside view controller already updated our instance variables by using the delegate
        saveSettings()
        
        //update our display (in case the user changed units)
        updateDisplayWithCurrentReadings()
    }
    
    
    @IBAction func updateButtonTapped(sender: UIButton) {
        performRestCall()
    }
    
    
    func performRestCall() {
        
      var url = NSURL(string: theURL)

      if (url == nil || theURL == "") {
        showAlert("Invalid or no URL entered!")
        return
      }

      updateButton.enabled = false

      var request = NSMutableURLRequest(URL: NSURL(string: theURL)!)
      request.HTTPMethod = "GET"
      request.setValue("application/json; charset=utf=8", forHTTPHeaderField: "Content-Type")
      request.setValue("application/json", forHTTPHeaderField: "Accept")
      request.cachePolicy = NSURLRequestCachePolicy.ReloadIgnoringLocalAndRemoteCacheData

      NSURLConnection.sendAsynchronousRequest(request, queue: NSOperationQueue(),
        completionHandler:{ (response:NSURLResponse!, data: NSData!, error: NSError!) -> Void in

          dispatch_async(dispatch_get_main_queue(), {
            if let anError = error {
              //error getting data
              self.showAlert(error!.localizedDescription)
            } else {
              //process JSON
              self.handleResultsOfWebCall(data)
            }
            self.updateButton.enabled = true
          })
          return
      })

    }
    
    
    func handleResultsOfWebCall(theData : NSData) {
    
      var error : NSError?

      var jsonResults = NSJSONSerialization.JSONObjectWithData(theData, options: nil, error: &error) as! NSDictionary
      if let jsonError = error  {
        println("JSON Error \(error!.localizedDescription)")
        showAlert("Error retrieving results-check the URL or Server!")
      } else {
        let theSensors = jsonResults["arduino"] as! NSArray
        for i in 0..<theSensors.count {
          let sensor = theSensors[i] as! NSDictionary
          let location = sensor["location"] as! String
          let temperature = sensor["celsius"] as! NSString

          if (location == "indoor") {
            indoorTemperatureCelsius = temperature.floatValue
          } else {
            outdoorTemperatureCelsius = temperature.floatValue
          }
        }
        lastUpdate = NSDate() 
        saveSettings()  
        updateDisplayWithCurrentReadings()
      }

    }
    
    
    func showAlert(alertMessage : String) {
        let alertController = UIAlertController(title: "Error", message: alertMessage, preferredStyle: .Alert)
        let okAction = UIAlertAction(title: "OK", style: .Default, handler: nil)
        alertController.addAction(okAction)
        self.presentViewController(alertController, animated: true, completion: nil)
        return
    }
    
    
    func updateDisplayWithCurrentReadings() {
        
        var tempIndoor = indoorTemperatureCelsius
        var tempOutdoor = outdoorTemperatureCelsius
        
        self.interiorTemperature?.font = UIFont(name: "SFDigitalReadout-HeavyOblique", size: 48)
        self.exteriorTemperature?.font = UIFont(name: "SFDigitalReadout-HeavyOblique", size: 48)
        
        if (indoorTemperatureCelsius == INVALID_TEMPERATURE) {
            self.interiorTemperature?.text = "___._°"
            self.interiorTemperature?.textColor = UIColor.redColor()
            
            self.exteriorTemperature?.text = "___._°"
            self.exteriorTemperature?.textColor = UIColor.redColor()
            
            self.lastUpdatedLabel?.text = "None Read"    //ignore date field and show notice
            return
        }
        
        
        if (self.displayAsCelsius == false) {
            //convert to fahrenheit
            tempIndoor = ((9.0 / 5.0) * indoorTemperatureCelsius) + 32.0
            tempOutdoor = ((9.0 / 5.0) * outdoorTemperatureCelsius) + 32.0
        }
        
        self.interiorTemperature?.text = String(format: "%.01f°", tempIndoor)
        setFieldColor(self.interiorTemperature!, OnCelsiusTemperature: indoorTemperatureCelsius)
        
        self.exteriorTemperature?.text = String(format: "%.01f°", tempOutdoor)
        setFieldColor(self.exteriorTemperature!, OnCelsiusTemperature: outdoorTemperatureCelsius)
        
        let lastUpdatedDateTime = formatLastUpdatedDate(lastUpdate)
        self.lastUpdatedLabel?.text = String(format:"Updated %@",lastUpdatedDateTime)
        
    }
    
    
    
    func setFieldColor(labelField : UILabel, OnCelsiusTemperature temperatureCelsius : Float) {
        if (temperatureCelsius < 7.2) {
            //cold so use dark blue
            labelField.textColor = UIColor.blueColor()
            return
        }
        if (temperatureCelsius < 15.5) {
            labelField.textColor = UIColor.cyanColor()
            return
        }
        if (temperatureCelsius < 21.1) {
            labelField.textColor = UIColor.yellowColor()
            return
        }
        if (temperatureCelsius < 32.3) {
            labelField.textColor = UIColor.orangeColor()
            return
        }
        
        labelField.textColor = UIColor.redColor()
        return
    }
    
    
    func formatLastUpdatedDate(theDate : NSDate) -> String {
        //return a formatted date in the local locale of the user
        let locale = NSLocale.currentLocale()
        let dateFormat = NSDateFormatter.dateFormatFromTemplate("MMMM d yyyy HH mm", options: 0, locale: locale)
        let dateFormatter = NSDateFormatter()
        dateFormatter.dateFormat = dateFormat
        
        return dateFormatter.stringFromDate(theDate)
    }
    
    
    //The following handle user settings and saves
    
    func saveSettings() {
        //save the user's settings in a plist file in the Documents directory for this application
        var documentsPath = applicationDocumentsDirectory()
        var documentsURL = documentsPath.URLByAppendingPathComponent(SETTINGS_FILENAME)
        
        var indoorTemperature = indoorTemperatureCelsius as NSNumber
        var outdoorTemperature = outdoorTemperatureCelsius as NSNumber
        var displayCelsius = displayAsCelsius as NSNumber
        
        var savedValues : NSDictionary = [ "indoor": indoorTemperature,
            "outdoor": outdoorTemperature,
            "lastReading": self.lastUpdate,
            "displayAsCelsius": displayCelsius,
            "theURL": self.theURL]
        
        savedValues.writeToURL(documentsURL, atomically: true)
        println(String(format:"Settings saved to %@", documentsURL))
    }
    
    
    func loadSettings() ->Bool {
        //load the user's settings in a plist file in the Documents directory for this application
        var documentsPath = applicationDocumentsDirectory()
        var documentsURL = documentsPath.URLByAppendingPathComponent(SETTINGS_FILENAME)
        
        var readValues = NSDictionary(contentsOfURL: documentsURL)
        
        
        if let dictValues = readValues {
            //restore the values
            self.indoorTemperatureCelsius = (dictValues["indoor"] as! NSNumber).floatValue
            self.outdoorTemperatureCelsius = (dictValues["outdoor"] as! NSNumber).floatValue
            self.lastUpdate = dictValues["lastReading"]as! NSDate
            self.displayAsCelsius = dictValues["displayAsCelsius"] as! NSNumber.BooleanLiteralType
            self.theURL = dictValues["theURL"] as! String
            return true
        }
        
        //if here no values ever been saved or retrieved, set up defaults and save them first time
        self.indoorTemperatureCelsius = INVALID_TEMPERATURE
        self.outdoorTemperatureCelsius = INVALID_TEMPERATURE
        self.lastUpdate = NSDate()  //set to current date and time
        self.displayAsCelsius = true //default to Celsius
        println("No settings found...Saving default values.")
        saveSettings()
        return false
    }
    
    
    func applicationDocumentsDirectory() -> NSURL {
        // Returns the URL path to the application's Documents directory.
        let documentDirectoryURL = NSFileManager.defaultManager().URLsForDirectory(.DocumentDirectory, inDomains: .UserDomainMask).last as! NSURL
        return documentDirectoryURL
    }
    
    
}

